// =============================================================================
// Parent   : BBoyShare.B_NavigationPoint
// Package  : BBoyShare
// Class    : B_MonsterSpawner
//
// Author   : Boom-Boy
// Homepage : http://mth.or.tp/
//
// Version  : 1.0
// =============================================================================

class B_MonsterSpawner extends B_NavigationPoint;

var(MonsterSpawner) class<ScriptedPawn> CreatureType;
var(MonsterSpawner) float TriggeredSpawnDeray;
var(MonsterSpawner) bool bSpawnOnceOnly;
var(MonsterSpawner) bool bSpawnEffect;
var(MonsterSpawner) int MaxMonster;
var(MonsterSpawnerEvent) bool bFinishedEvent;
var(MonsterSpawnerEvent) name FinishedEventTag;
var(MonsterSpawnerEvent) float FinishedEventDerayTime;
var int NumMonster;

var(SpawnerOrders) name MS01_Orders;
var(SpawnerOrders) name MS02_OrderTag;

var(SpaenerCombat) float MS01_SightRadius;
var(SpaenerCombat) float MS02_Skill;
var(SpaenerCombat) float MS03_Aggressiveness;
var(SpaenerCombat) float MS04_CombatStyle;
var(SpaenerCombat) float MS05_ReFireRate;
var(SpaenerCombat) class<Projectile> MS06_RangedProjectile;
var(SpaenerCombat) float MS07_ProjectileSpeed;

var(SpawnerStatus) int MS01_Health;
var(SpawnerStatus) float MS02_AccelRate;
var(SpawnerStatus) float MS03_GroundSpeed;
var(SpawnerStatus) float MS04_AirSpeed;
var(SpawnerStatus) float MS05_JumpZ;
var(SpawnerStatus) float MS06_WaterSpeed;
var(SpawnerStatus) bool bSpawnFalling;

simulated function PostBeginPlay() {
	
	local B_MonsterSpawner Spawner;
	
	super.PostBeginPlay();
	
	NumMonster = 0;
	
	if (CreatureType == None) {
		bFinishedEvent = false;
		GotoState('Finished');
	}
}

Auto State Waiting
{
	function Trigger(Actor Other, Pawn EventInstigator) {
		
		local Actor A;
		
		if (Other.Event == Tag && Other.Tag == Name) {
			NumMonster = Max(NumMonster - 1, 0);
			return;
		}
		
		if (Event != '') {
			ForEach AllActors(class'Actor', A, Event)
				A.Trigger(self, EventInstigator);
		}
		
		Instigator = EventInstigator;
		
		GotoState('Spawning');
	}
}

State Spawning
{
	function Timer() {
		
		if (TrySpawn() && bSpawnOnceOnly)
			GotoState('Finished');
		else
			GotoState('Waiting');
	}
	
	function Trigger(Actor Other, Pawn EventInstigator) {
		
		if (Other.Event == Tag && Other.Tag == Name) {
			Instigator = EventInstigator;
			NumMonster = Max(NumMonster - 1, 0);
		}
	}
	
	function bool TrySpawn() {
		
		local ScriptedPawn Monster;
		local Rotator NewRot;
		
		if (NumMonster >= MaxMonster)
			return false;
		
		Monster = Spawn(CreatureType);
		if (Monster == None)
			return false;
		
		NewRot = Rot(0, 0, 0);
		NewRot.Yaw = Rotation.Yaw;
		Monster.SetRotation(NewRot);
		
		if (bSpawnFalling)
			Monster.SetPhysics(PHYS_Falling);
		
		return SetStatus(Monster);
	}
	
	function bool SetStatus(ScriptedPawn Monster) {
		
		local UTTeleportEffect PTE;
		
		if (Monster == None)
			return false;
		
		Monster.RemoteRole = ROLE_SimulatedProxy;
		Monster.bCollideWorld = true;
		
		if (MS01_Orders != '')
			Monster.Orders = MS01_Orders;
		else
			Monster.GotoState('Attacking');
		if (MS02_OrderTag != '')
			Monster.OrderTag = MS02_OrderTag;
		if (MS01_SightRadius > 0)
			Monster.SightRadius = MS01_SightRadius;
		if (MS02_Skill > 0)
			Monster.Skill = MS02_Skill;
		if (MS03_Aggressiveness > 0)
			Monster.Aggressiveness = MS03_Aggressiveness;
		if (MS04_CombatStyle > 0)
			Monster.CombatStyle = MS04_CombatStyle;
		if (MS05_ReFireRate > 0)
			Monster.ReFireRate = MS05_ReFireRate;
		if (MS06_RangedProjectile != None)
			Monster.RangedProjectile = MS06_RangedProjectile;
		if (MS07_ProjectileSpeed > 0)
			Monster.ProjectileSpeed = MS07_ProjectileSpeed;
		if (MS01_Health > 0)
			Monster.Health = MS01_Health;
		if (MS02_AccelRate > 0)
			Monster.AccelRate = MS02_AccelRate;
		if (MS03_GroundSpeed > 0)
			Monster.GroundSpeed = MS03_GroundSpeed;
		if (MS04_AirSpeed > 0)
			Monster.AirSpeed = MS04_AirSpeed;
		if (MS05_JumpZ > 0)
			Monster.JumpZ = MS05_JumpZ;
		if (MS06_WaterSpeed > 0)
			Monster.WaterSpeed = MS06_WaterSpeed;
		
		Monster.Event = Tag;
		Monster.Tag = Name;
		
		Monster.SetMovementPhysics();
		
		if (bSpawnEffect) {
			PTE = Spawn(class'UTTeleportEffect', Monster, , Monster.Location, Monster.Rotation);
			PTE.PlaySound(sound'Resp2A', , 10.0);
		}
		
		NumMonster++;
		
		return true;
	}
	
	function BeginState() {
		
		// TODO
	}
	
Begin:
	if (TriggeredSpawnDeray > 0.0)
		SetTimer(TriggeredSpawnDeray, false);
	else
		Timer();
}

State Finished
{
	function ExecuteEvent() {
		
		local Actor A;
		
		if (FinishedEventTag != '') {
			ForEach AllActors(class'Actor', A, FinishedEventTag) {
				A.Trigger(self, Instigator);
			}
		}
	}
	
	function Timer() {
		
		ExecuteEvent();
	}
	
Begin:
	if (bFinishedEvent) {
		if (FinishedEventDerayTime > 0.0)
			SetTimer(FinishedEventDerayTime, false);
		else
			ExecuteEvent();
	}
}

defaultproperties
{
     CreatureType=Class'UnrealShare.Brute'
     MaxMonster=10
     MS01_Orders=Attacking
     bDirectional=True
     Texture=Texture'UnrealShare.S_SpawnP'
}
